#ifndef MULTIREADER_H
#define MULTIREADER_H

#include <Exception.h>
#include <map>

/// Read data from multiple filehandles
class MultiReader
{
public:
	/// Interface for data listeners
	class Listener
	{
	public:
		/// Notify that there is data to be read on `fd'
		virtual bool haveData(int fd) throw () { return true; }

		/// Notify the end-of-file condition on `fd'
		virtual bool haveEOF(int fd) throw () { return true; }
	};

	// Base class for line-oriented listeners
	class LineListener : public Listener
	{
	protected:
		std::string line;

	public:
		virtual ~LineListener() throw () {}

		virtual bool haveData(int fd) throw ();

		virtual bool haveLine(const std::string& line) throw () = 0;
	};

protected:
	std::map<int, Listener*> listeners;

public:
	/// Add another fd and listener to the listening set
	void addFD(int fd, Listener* l) throw ();

	/// Do the read loop; continue until a listener returns false or all file
	/// descriptors have been closed.  Returns true if exited because all file
	/// descriptors have been closed, false if exited because a Listener returned
	/// false in haveData
	bool readLoop() throw (SystemException);
};

// vim:set ts=4 sw=4:
#endif
